#include <windows.h>
#include <gl/glfw.h>
#include <math.h>

#include "Vector.h"

void glVertexV(Vector& vec)
{
	glVertex3f(vec.x, vec.y, vec.z);
}

int main()
{
	glfwInit();

	if(!glfwOpenWindow(800, 600, 0,0,0,0, 16, 0, GLFW_WINDOW)) // width, height, r, g, b, a, depth, stencil, mode (0 for r,g,b uses desktop settings)
	{
		glfwTerminate();
		return 0;
	}
	glfwSetWindowTitle("crazierbezier");

	glfwSwapInterval(1);

	glfwEnable(GLFW_STICKY_KEYS);

	glViewport(0, 0, 800, 600);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

	glDisable(GL_DEPTH_TEST);
	glDisable(GL_TEXTURE_2D);
	
	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_POINT_SMOOTH);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f, 800.0f/600.0f, 0.2f, 500.0f);
	glMatrixMode(GL_MODELVIEW);
	
	Vector cpts[100];
	Vector ipts[100];
	int numcpts;
	int numipts;

	// some random fu to set up control points
	
	numcpts=0;
	cpts[numcpts++]=Vector(-1.0f, 1.0f, 0);
	cpts[numcpts++]=Vector( 0.0f,-1.0f, 0);
	cpts[numcpts++]=Vector( 1.0f, 0.5f, 0);
	cpts[numcpts++]=Vector( 0.0f, 1.0f, 0);
	cpts[numcpts++]=Vector(-0.5f, 0.7f, 0);
	cpts[numcpts++]=Vector( 0.5f,-0.4f, 0);
	cpts[numcpts++]=Vector( 1.0f, 1.0f, 0);
	cpts[numcpts++]=Vector(-1.0f, 0.0f, 0);
	cpts[numcpts++]=Vector(-0.5f, 0.5f, 0);
	cpts[numcpts++]=Vector( 0.0f,-0.5f, 0);
	cpts[numcpts++]=Vector( 0.5f, 0.5f, 0);
	cpts[numcpts++]=Vector( 1.0f,-1.0f, 0);
	cpts[numcpts++]=Vector( 1.0f, 1.0f, 0);
	cpts[numcpts++]=Vector(-1.0f, 1.0f, 0);
	cpts[numcpts++]=Vector(-1.0f,-1.0f, 0);
	cpts[numcpts++]=Vector( 1.0f,-1.0f, 0);
	cpts[numcpts++]=Vector( 0.0f, 0.0f, 0);

	int tnum=numcpts;	
	for(int i=1;i<tnum;i++)
	{
		cpts[numcpts++].x=cpts[tnum-i-1].y;
		cpts[numcpts++].y=cpts[tnum-i-1].x;
		cpts[numcpts++].z=0;
	}

	cpts[numcpts++]=Vector( 1.0f,-1.0f, 0);
	cpts[numcpts++]=Vector( 1.0f, 1.0f, 0);
	cpts[numcpts++]=Vector(-1.0f, 1.0f, 0);
	cpts[numcpts++]=Vector(-1.0f,-1.0f, 0);
	cpts[numcpts++]=Vector( 1.0f,-1.0f, 0);
	
	cpts[27].x=0.3f;
	cpts[28].x=1.0f;
	cpts[29].y=-0.7f;

	// initialize bezier curve	
	Vector bezier[1000];
	int bezier_max=1000;
	int bezier_numpoints=0;
	for(int i=0;i<bezier_max;i++)
		bezier[i]=cpts[0];
	
	float a=0.0f;
	float t=0.0f;

	for(;;)
	{
		a+=0.002f;
		t=0.5f-(float)cos(a)*0.5f;

		if(glfwGetKey(GLFW_KEY_ESC) || !glfwGetWindowParam(GLFW_OPENED))
			break;

		glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

		glLoadIdentity();
		glTranslatef(0,0,-2.5f);

		// draw control polygon
		glLineWidth(3);
		glColor3f(0.35f, 0.2f, 0.2f);
		glBegin(GL_LINES);
		for(int i=1;i<numcpts;i++)
		{
			glVertexV(cpts[i-1]);
			glVertexV(cpts[i]);
		}
		glEnd();

		// calculate and draw successively higher-level interpolated polygons/curves
		glLineWidth(2);
		glBegin(GL_LINES);
		numipts=numcpts;
		for(int i=0;i<numipts;i++)
			ipts[i]=cpts[i];
		do
		{
			float color=1.0f-(float)numipts/numcpts;
			numipts--;
			glColor3f(0.4f+0.9f*color, 0.0f+0.8f*color, 0.0f+0.6f*color);
			for(int i=0;i<numipts;i++)
			{
				ipts[i]=ipts[i]+(ipts[i+1]-ipts[i])*t;
				if(i>0)
				{
					glVertexV(ipts[i-1]);
					glVertexV(ipts[i]);
				}
			}
		} while(numipts>1);
		glEnd();

		// update latest highest-level point
		bezier[bezier_numpoints]=ipts[0];

		// draw bezier curve
		glLineWidth(2);
		glBegin(GL_LINES);
		int pi=bezier_numpoints+2;
		for(int i=0;i<bezier_max-1;i++)
		{
			float fi=(float)i/bezier_max;
			glColor3f(1.0f*fi, 1.2f*fi, 1.7f*fi);
			glVertexV(bezier[(pi-1+bezier_max)%bezier_max]);
			glVertexV(bezier[pi%bezier_max]);
			
			pi++;
		}
		glEnd();

		// add points to bezier curve
		if(bezier_numpoints<bezier_max && (bezier_numpoints==0 || bezier[bezier_numpoints-1]/bezier[bezier_numpoints]>10.0f/800))
		{
			bezier_numpoints++;
			bezier_numpoints%=bezier_max;
		}

		// draw latest point
		glColor3f(1,1,1);
		glPointSize(6);
		glBegin(GL_POINTS);
		glVertexV(ipts[0]);
		glEnd();

		glfwSwapBuffers();
	}

	return 0;
}

